//! Chatbot
use std::convert::TryInto;

use anyhow::Result;
use matrix_sdk::{
    self, async_trait,
    events::{
        room::message::{MessageEventContent, MessageType, TextMessageEventContent},
        AnyMessageEventContent, SyncMessageEvent,
    },
    room::Room,
    Client, ClientConfig, EventHandler, SyncSettings,
};

use crate::Html;

#[derive(Debug, Clone, Default)]
pub(crate) struct Builder {
    user: String,
    password: String,
    homeserver: String,
    //TODO: rooms
    room: String,
}

impl Builder {
    fn new() -> Self {
        Default::default()
    }

    pub(crate) async fn connect(&self) -> Result<Chatbot> {
        let client = Client::new(self.homeserver.as_str())?;
        client
            .login(self.user.as_str(), self.password.as_str(), None, None)
            .await?;
        assert!(client.logged_in().await);
        client
            .join_room_by_id(&self.room.as_str().try_into()?)
            .await?;

        Ok(Chatbot { client })
    }

    pub(crate) fn login(
        &mut self,
        user: &impl AsRef<str>,
        password: &impl AsRef<str>,
    ) -> &mut Self {
        self.user = String::from(user.as_ref());
        self.password = String::from(password.as_ref());
        self
    }

    pub(crate) fn homeserver(&mut self, homeserver: &impl AsRef<str>) -> &mut Self {
        self.homeserver = String::from(homeserver.as_ref());
        self
    }

    pub(crate) fn room(&mut self, room: &impl AsRef<str>) -> &mut Self {
        self.room = String::from(room.as_ref());
        self
    }
}

#[derive(Debug, Clone)]
pub(crate) struct Chatbot {
    client: Client,
}

impl Chatbot {
    pub(crate) fn builder() -> Builder {
        Builder::new()
    }

    pub(crate) async fn run(&self) -> Result<()> {
        self.client.set_event_handler(Box::new(Html::new())).await;

        let mut settings = SyncSettings::default();
        if let Some(token) = self.client.sync_token().await {
            settings = settings.token(token);
        }
        self.client.sync(settings).await;
        Ok(())
    }
}
