use std::convert::TryInto;
use std::env;

use anyhow::Result;
use log::info;
use matrix_sdk::{
    self, async_trait,
    events::{
        room::message::{MessageEventContent, MessageType, TextMessageEventContent},
        AnyMessageEventContent, SyncMessageEvent,
    },
    room::Room,
    Client, ClientConfig, EventHandler, SyncSettings,
};

use crieur_retrieve::{ArticleLocation, Url};

pub(crate) struct Html {}

impl Html {
    pub fn new() -> Self {
        Self {}
    }
}

async fn send_article<U, E>(url: U, room: matrix_sdk::room::Joined)
where
    U: TryInto<Url, Error = E> + Send,
    E: std::error::Error + Sync + Send + 'static,
{
    //TODO: replace by async block when async block is stable
    async fn article_html<U, E>(url: U) -> Result<String>
    where
        U: TryInto<Url, Error = E> + Send,
        E: std::error::Error + Sync + Send + 'static,
    {
        let article_str = ArticleLocation::builder()
            .url(url)?
            .build()?
            .retrieve_html()
            .await?;
        Ok(article_str)
    }

    let text_message =
        |message| AnyMessageEventContent::RoomMessage(MessageEventContent::text_plain(message));

    //TODO: replace occurences ok() by async and logging block when async block is stable
    let article_html = match article_html(url).await {
        Ok(url) => url,
        Err(_) => {
            room.send(text_message("Can't download the file"), None)
                .await
                .ok();
            return;
        }
    };

    room.send_attachment(
        "test.html",
        &mime::TEXT_HTML_UTF_8,
        &mut article_html.as_bytes(),
        None,
    )
    .await
    .ok();
}

#[async_trait]
impl EventHandler for Html {
    async fn on_room_message(&self, room: Room, event: &SyncMessageEvent<MessageEventContent>) {
        if let Room::Joined(room) = room {
            let msg_body = if let SyncMessageEvent {
                content:
                    MessageEventContent {
                        msgtype: MessageType::Text(TextMessageEventContent { body: msg_body, .. }),
                        ..
                    },
                ..
            } = event
            {
                msg_body
            } else {
                return;
            };
            info!("sending file");

            match msg_body.split(' ').collect::<Vec<_>>().as_slice() {
                ["!html", url, ..] => send_article(*url, room).await,
                _ => return,
            }
        }
    }
}
